﻿using MassFoster;
using ShapeVolume;
using Movement;
using SharpWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;
using Variables;
using System.Data;
using EntitySpawn;

namespace CoreImplement
{
    public class SystemHost : WsSystemHost
    {
        private UInt128 current_entity_id  = 5;
        public UInt128 AllocNewID()
        {
            lock (this)
            {
                return current_entity_id++;
            }
        }

        private Dictionary<string, WsBaseSystem> _system_store = new Dictionary<string, WsBaseSystem>();

        public WsSystem<Datas> GetSystem<Datas, Sys>() where Datas : WsItemParams where Sys : WsSystem<Datas>
        {
#pragma warning disable CS8603 // 可能返回 null 引用。
            var type_name = typeof(WsSystem<Datas>).FullName;
            if (type_name == null)
                throw new ArgumentException("类型名称为null");
            return _system_store[type_name] as WsSystem<Datas>;
#pragma warning restore CS8603 // 可能返回 null 引用。
        }

        public void RegisterSystem<Datas>(WsSystem<Datas> inst) where Datas : WsItemParams
        {
#pragma warning disable CS8604 // 引用类型参数可能为 null。
            _system_store[typeof(WsSystem<Datas>).FullName] = inst;
#pragma warning restore CS8604 // 引用类型参数可能为 null。
        }

        private List<WsExecutableSystem> ExecutableSystemList
        {
            get
            {
                var items = from n in _system_store.Values
                            orderby n.Name
                            where n is WsExecutableSystem
                            select n as WsExecutableSystem;
                return items.ToList();
            }
        }

        private List<Tuple<WsExecutableSystem, UInt128>> execute_list = new List<Tuple<WsExecutableSystem, UInt128>>();
        public void LoopRun()
        {
            var task_list = ExecutableSystemList;
            foreach (var t in task_list)
            {
                execute_list.Add(new Tuple<WsExecutableSystem, UInt128>(t, 0));
            }

            for (; ; )
            {
                var unit = execute_list.ElementAt(0);
                unit.Item1.Run(this, unit.Item2);
                execute_list.RemoveAt(0);

                bool find_flags = false;
                var next_time = unit.Item2 + unit.Item1.ExecutePeriodMsec;
                for (var tidx = 0; tidx < execute_list.Count; ++tidx)
                {
                    var item = execute_list.ElementAt(tidx);

                    if (next_time < item.Item2)
                    {
                        execute_list.Insert(tidx, new Tuple<WsExecutableSystem, UInt128>(unit.Item1, next_time));
                        find_flags = true;
                        break;
                    }
                }

                if (!find_flags)
                {
                    execute_list.Add(new Tuple<WsExecutableSystem, UInt128>(unit.Item1, next_time));
                }
            }
        }
    }

    public class ExampleEntity : WsEntity, ShapeVolumeParams, MovementParam, MassFosterParams, LifetimeParams
    {
        public string TypeName { get; set; } = "示例实体";
        public UInt128 RuntimeID { get; set; } = 1;
        public string JsonContent { get => throw new NotImplementedException(); set => throw new NotImplementedException(); }

        public bool IsConsumed { get; set; } = true;
        public double AbsorbAmount { get; set; } = 0;
        public double MassInput { get; set; } = 0;
        public VariableDouble MassConvRates { get; } = new VariableDouble("重量转换效率", 0.0001);


        public List<VariableVal> TotalVariables { get; } = new List<VariableVal>();
        public WsBaseSystem? BindSystem { get; set; } = null;
        public WsEntity BindEntity => this;

        public Vector3 PosXYZ { get; set; } = new Vector3();
        public Vector3 OritXYZ { get; set; } = new Vector3(1, 0, 0);
        public float Speed { get; set; } = 0.01f;
        public double Length { get; set; }
        public double Width { get; set; }
        public double Height { get; set; }
        private VariableDouble volume_current = new VariableDouble("", 0, 0, double.MaxValue / 2);
        public double Volume { get { return volume_current.Value; } set { volume_current.Value = value; } }

        public VariableDouble PrimitiveLength { get; } = new VariableDouble("初生体长", 1);
        public VariableDouble PrimitiveWidth { get; } = new VariableDouble("初生体宽", 0.5);
        public VariableDouble PrimitiveHeight { get; } = new VariableDouble("初生体高", 0.5);

        public VariableDouble Density { get; set; } = new VariableDouble("生体密度", 1);
        public UInt128 SpawnStart { get; set; }
        public UInt128 LifetimeAcc { get; set; }
        public double MatureVolume { get; set; } = -1;

        public VariableULong GrowthTimeCount { get; } = new VariableULong("成长期长度", 1_000_000);
        public VariableULong MatureTimeCount { get; } = new VariableULong("成熟期长度", 100_000_000_000);
        public VariableULong DeathTimePoint { get; } = new VariableULong("死亡时间点", 120_000_000_000);
        public VariableULong SpawnPeriods { get; } = new VariableULong("繁育周期", 100_000);

        public uint SpawnTimes { get; set; } = 0;

        public VariablePercent ImpressRates { get; } = new VariablePercent("能量吸收转换效率", 0.1);

        public void ActiveConfigBeforeExecute(WsSystemHost host)
        {
            var mass_sys = host.GetSystem<MassFosterParams, EnvFosterSystem>();
            var vol_sys = host.GetSystem<ShapeVolumeParams, ShapeVolumeSystem>();
            var mov_sys = host.GetSystem<MovementParam, MovementSystem>();
            var spawn_sys = host.GetSystem<LifetimeParams, LifetimeSystem>();

            mass_sys.RegisterItem(this);
            vol_sys.RegisterItem(this);
            mov_sys.RegisterItem(this);
            spawn_sys.RegisterItem(this);

            Console.WriteLine($"实体诞生：ID={RuntimeID}");
        }

        public void DeathExecute(WsSystemHost host)
        {
            var mass_sys = host.GetSystem<MassFosterParams, EnvFosterSystem>();
            var vol_sys = host.GetSystem<ShapeVolumeParams, ShapeVolumeSystem>();
            var mov_sys = host.GetSystem<MovementParam, MovementSystem>();
            var spawn_sys = host.GetSystem<LifetimeParams, LifetimeSystem>();

            mass_sys.RemoveItem(this.RuntimeID);
            vol_sys.RemoveItem(this.RuntimeID);
            mov_sys.RemoveItem(this.RuntimeID);
            spawn_sys.RemoveItem(this.RuntimeID);

            Console.WriteLine($"实体死亡：ID={RuntimeID}");
        }

        public ExampleEntity(UInt128 runtime_id)
        {
            this.RuntimeID = runtime_id;
            TotalVariables.Add(MassConvRates);
            TotalVariables.Add(PrimitiveLength);
            TotalVariables.Add(PrimitiveWidth);
            TotalVariables.Add(PrimitiveHeight);
            TotalVariables.Add(Density);
            TotalVariables.Add(GrowthTimeCount);
            TotalVariables.Add(MatureTimeCount);
            TotalVariables.Add(DeathTimePoint);
            TotalVariables.Add(SpawnPeriods);

            Length = PrimitiveLength.Value;
            Width = PrimitiveWidth.Value;
            Height = PrimitiveHeight.Value;
            Volume = Length * Width * Height;
        }

        public WsEntity NewEmpty(UInt128 id)
        {
            return new ExampleEntity(id);
        }

    }

}
