﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ShapeVolume;
using SharpWorld;
using Variables;

namespace MassFoster
{
    public interface MassFosterParams : WsItemParams
    {
        bool IsConsumed { get; set; }

        double AbsorbAmount { get; set; }
        double MassInput { get; set; }
        /// <summary>
        /// 光合作用转换效率
        /// </summary>
        VariableDouble MassConvRates { get; }
    }

    public class EnvFosterSystem : WsSystem<MassFosterParams>, WsExecutableSystem
    {
        private Dictionary<UInt128, MassFosterParams> _items_store = new Dictionary<UInt128, MassFosterParams>();

        public string Name => "环境能量养护系统";

        public uint ExecutePeriodMsec => 1000;

        public MassFosterParams GetItem(UInt128 RtID)
        {
            lock (this)
            {
                return _items_store[RtID];
            }
        }

        public void RegisterItem(MassFosterParams entity_params)
        {
            lock (this)
            {
                _items_store[entity_params.BindEntity.RuntimeID] = entity_params;
            }
        }

        private void update_batch(WsSystemHost host, MassFosterParams item)
        {
            var volsys = host.GetSystem<ShapeVolumeParams, ShapeVolumeSystem>();
            var volparam = volsys.GetItem(item.BindEntity.RuntimeID);
            var square = volparam.Length * volparam.Width;

            if (item.IsConsumed)
            {
                item.IsConsumed = false;
                item.AbsorbAmount = 0;
                item.MassInput = 0;
            }

            item.AbsorbAmount += _energy_density * square;
            var new_mass = item.AbsorbAmount * item.MassConvRates.Value;
            item.MassInput = new_mass;
        }

        public void Run(WsSystemHost host, UInt128 timepoint)
        {
            var srcl = _items_store.Values.ToList();
            var opts = new ParallelOptions { MaxDegreeOfParallelism = Environment.ProcessorCount };
            Parallel.For(0, srcl.Count, opts, index => {
                var param = srcl[index];
                update_batch(host, param);
            });
        }

        public void RemoveItem(UInt128 RtID)
        {
            lock (this)
            {
                _items_store.Remove(RtID);
            }
        }

        //=========================
        /// <summary>
        /// 表面能量密度
        /// </summary>
        private double _energy_density = 0;
        /// <summary>
        /// 构建能量消耗计算系统
        /// </summary>
        /// <param name="plain_energy"></param>
        public EnvFosterSystem(double plain_energy)
        {
            _energy_density = plain_energy;
        }

    }
}
