﻿using SharpWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;

namespace Movement
{
    public interface MovementParam : WsItemParams
    {
        public Vector3 PosXYZ { get; set; }
        public Vector3 OritXYZ { get; set; }

        public float Speed { get; set; }
    }

    public class MovementSystem : WsSystem<MovementParam>, SharpWorld.WsExecutableSystem
    {
        private Dictionary<UInt128, MovementParam> _items_store = new Dictionary<UInt128, MovementParam>();

        public string Name => "机动演算系统";

        public uint ExecutePeriodMsec => 100;

        public MovementParam GetItem(UInt128 RtID)
        {
            lock (this)
            {
                return _items_store[RtID];
            }
        }

        public void RegisterItem(MovementParam entity_params)
        {
            lock (this)
            {
                _items_store[entity_params.BindEntity.RuntimeID] = entity_params;
            }
        }

        public void movement_update(WsSystemHost host, MovementParam item_ps)
        {
            var orit_vec = Vector3.Normalize(item_ps.OritXYZ);
            var move_span = Vector3.Multiply(orit_vec, item_ps.Speed);
            item_ps.PosXYZ = Vector3.Add(item_ps.PosXYZ, move_span);
        }

        public void Run(WsSystemHost host, UInt128 timepoint)
        {
            var srcl = _items_store.Values.ToList();
            var opts = new ParallelOptions { MaxDegreeOfParallelism = Environment.ProcessorCount };
            Parallel.For(0, srcl.Count, opts, index => {
                var it = srcl[index];
                movement_update(host, it);
            });
        }

        public void RemoveItem(UInt128 RtID)
        {
            lock (this)
            {
                _items_store.Remove(RtID);
            }
        }
    }
}
