﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MassFoster;
using SharpWorld;
using Variables;

namespace ShapeVolume
{
    public interface ShapeVolumeParams : WsItemParams
    {
        public double Length { get; set; }
        public double Width { get; set; }
        public double Height { get; set; }
        public double Volume { get; set; }

        public VariableDouble PrimitiveLength { get; }
        public VariableDouble PrimitiveWidth { get; }
        public VariableDouble PrimitiveHeight { get; }
        public VariableDouble Density { get; }

        public VariablePercent ImpressRates { get; }
    }

    public class ShapeVolumeSystem : WsExecutableSystem, WsSystem<ShapeVolumeParams>
    {
        private Dictionary<UInt128, ShapeVolumeParams> _items_store = new Dictionary<UInt128, ShapeVolumeParams>();

        public uint ExecutePeriodMsec => 10_000;

        public string Name => "体重演算系统";

        public ShapeVolumeParams GetItem(UInt128 RtID)
        {
            lock (this)
            {
                return _items_store[RtID];
            }
        }

        public void RegisterItem(ShapeVolumeParams entity_params)
        {
            lock (this)
            {
                _items_store[entity_params.BindEntity.RuntimeID] = entity_params;
            }
        }

        private void update_volumes(WsSystemHost host, ShapeVolumeParams vmit)
        {
            var mass_sys = host.GetSystem<MassFosterParams, EnvFosterSystem>();

            var mass_param = mass_sys.GetItem(vmit.BindEntity.RuntimeID);
            mass_param.IsConsumed = true;
            var m_new = mass_param.MassInput;
            var vol_new = m_new / vmit.Density.Value * vmit.ImpressRates.Value;
            vmit.Volume += vol_new;

            var primitive_vol = vmit.PrimitiveHeight.Value * vmit.PrimitiveLength.Value * vmit.PrimitiveWidth.Value;
            var times = Math.Pow(vmit.Volume / primitive_vol, 1 / 3.0);
            vmit.Length = times * vmit.PrimitiveLength.Value;
            vmit.Width = times * vmit.PrimitiveWidth.Value;
            vmit.Height = times * vmit.PrimitiveHeight.Value;
        }

        public void Run(WsSystemHost host, UInt128 timepoint_msec)
        {
            var srcl = _items_store.Values.ToList();
            var opts = new ParallelOptions { MaxDegreeOfParallelism = Environment.ProcessorCount };
            Parallel.For(0, srcl.Count, opts, index => {
                var it = srcl[index];
                update_volumes(host, it);
            });
        }

        public void RemoveItem(UInt128 RtID)
        {
            lock (this)
            {
                _items_store.Remove(RtID);
            }
        }
    }
}
