﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading.Tasks;

namespace Variables
{
    public abstract class VariableData<D> : SharpWorld.VariableVal
        where D : struct
    {
        public string Name { get; private set; }
        private D value_store;
        public virtual D Value
        {
            get
            {
                return value_store;
            }
            set
            {
                value_store = value;
            }
        }

        public VariableData(string name)
        {
            this.Name = name;
        }

        public abstract void Variation(double rates);
    }

    public class VariableDouble : VariableData<Double>
    {
        public double Min { get; set; } = -Double.MaxValue;
        public double Max { get; set; } = Double.MaxValue;

        public VariableDouble(string name) : base(name) { }
        public VariableDouble(string name, double val) : base(name) { Value = val; }
        public VariableDouble(string name, double val, double min, double max)
            : base(name)
        {
            Value = val;
            Min = min;
            Max = max;
        }

        public override double Value
        {
            get { return base.Value; }
            set
            {
                base.Value = Math.Max(Min, Math.Min(Max, value));
            }
        }

        public override void Variation(double rates)
        {
            var dtinst = DateTime.Now.Millisecond;
            var flag = dtinst % 2;
            Value *= (1 + Math.Pow(-1, flag) * rates);
        }
    }

    public class VariablePercent : VariableDouble
    {
        public VariablePercent(string name) : base(name) { }
        public VariablePercent(string name, double val) : base(name, val)
        {
            Value = val;
        }

        public override double Value
        {
            get
            {
                return base.Value;
            }
            set
            {
                base.Value = Math.Min(1, Math.Max(0, value));
            }
        }
    }

    public class VariableInt : VariableData<int>
    {
        public VariableInt(string name) : base(name) { }

        public override void Variation(double rates)
        {
            var dtinst = DateTime.Now.Millisecond;
            var flag = dtinst % 2;
            var temp_val = Value * (1 + Math.Pow(-1, flag) * rates);
            Value = (int)temp_val;
        }
    }


    public class VariableULong : VariableData<UInt64>
    {
        public VariableULong(string name, ulong val) : base(name) { Value = val; }

        public override void Variation(double rates)
        {
            var dtinst = DateTime.Now.Millisecond;
            var flag = dtinst % 2;
            var temp_val = Value * (1 + Math.Pow(-1, flag) * rates);
            Value = (UInt64)temp_val;
        }
    }
}
